// Oleg Petruny proprietary.


#include "AgeOfWarManager.h"

#include "Camera/CameraComponent.h"

#include "AgeOfWarUnit.h"
#include "MainGameModeBase.h"
#include "PlayerBase.h"

TMap<UClass*, FAgeOfWarUnitStats> AAgeOfWarManager::unitStats = {};

const FAgeOfWarUnitStats& AAgeOfWarManager::UnitGetStats(TSubclassOf<AAgeOfWarUnit> unitClass)
{
    return unitStats[unitClass];
}

AAgeOfWarManager::AAgeOfWarManager()
{
    auto root = CreateDefaultSubobject<USceneComponent>(TEXT("Scene"));

    camera = CreateDefaultSubobject<UCameraComponent>(TEXT("Camera"));
    camera->AttachToComponent(root, FAttachmentTransformRules::KeepRelativeTransform);

    basePlayer = static_cast<AAgeOfWarUnit*>(CreateDefaultSubobject(TEXT("BasePlayer"), AAgeOfWarUnit::StaticClass(), baseClass, true, false));
    baseComputer = static_cast<AAgeOfWarUnit*>(CreateDefaultSubobject(TEXT("BaseComputer"), AAgeOfWarUnit::StaticClass(), baseClass, true, false));
}

void AAgeOfWarManager::Start(APlayerBase* playerPawn, FMinigameEndCallback delegate)
{
    if(!playerPawn)
        return;

    AMinigame::Start(playerPawn, delegate);

    player->LockPlayer(FPlayerLock::All());
    player->SwitchToView(this);

    FillUnitStats();
}

void AAgeOfWarManager::End()
{
    unitStats.Empty();

    player->ReturnPlayerView();
    player->UnlockPlayer(FPlayerLock::All());

    AMinigame::End();
}

void AAgeOfWarManager::UnitKill(AAgeOfWarUnit* unit)
{}

void AAgeOfWarManager::AddUnitStats(TSubclassOf<AAgeOfWarUnit> unitClass, FAgeOfWarUnitStats stats)
{
    unitStats.Add(unitClass, stats);
}
