// Oleg Petruny proprietary.


#include "AgeOfWarUnit.h"

#include "AgeOfWarManager.h"

const FAgeOfWarUnitStats& AAgeOfWarUnit::GetStats()
{
    if(!stats)
        stats = &AAgeOfWarManager::UnitGetStats(GetClass());

    if(ensureAlways(stats))
    {
        auto errorMsg = FString::Printf(TEXT("AgeOfWarManager instance doesn't contain %s class stats... Destroying actor"), &GetClass()->GetName());
        UE_LOG(LogTemp, Log, TEXT("%s"), *errorMsg);
        GEngine->AddOnScreenDebugMessage(-1, 20.0f, FColor::Red, errorMsg);
        static FAgeOfWarUnitStats s{};
        stats = &s;
        Destroy();
    }

    return *stats;
}

void AAgeOfWarUnit::BeginPlay()
{
    health = GetStats().health;
}

void AAgeOfWarUnit::EndPlay(const EEndPlayReason::Type EndPlayReason)
{}

void AAgeOfWarUnit::Tick(float deltaTime)
{
    Super::Tick(deltaTime);

    if(GetStats().moveSpeed <= 0)
        return;

    // move unit forward
    auto moveStep = GetActorForwardVector() * GetStats().moveSpeed;
    FHitResult hit;
    this->AddActorWorldOffset(moveStep, true, &hit, ETeleportType::None);

    // check if blocked
    if(!hit.bBlockingHit)
        return;

    forwardUnit = Cast<AAgeOfWarUnit>(hit.GetActor());
    if(!forwardUnit) // something is wrong, only block with units
    {
        Destroy();
        return;
    }

    // iterate to attackable unit
    for(int i = 1; forwardUnit->team == team && i < GetStats().attackDistance; i += forwardUnit->GetStats().unitSize)
        forwardUnit = forwardUnit->forwardUnit;

    // if forward unit is in the same team skip attack
    if(forwardUnit->team == team)
        return;

    // attack the forward unit
    if(lastAttackTimestamp + GetStats().attackRate - FPlatformTime::Seconds() < 0)
    {
        lastAttackTimestamp = FPlatformTime::Seconds();
        forwardUnit->Damage(GetStats().attackDamage);
    }
}

void AAgeOfWarUnit::Damage(const int32 damage)
{
    health -= damage;

    if(health > 0)
        return;

    if(IsValid(manager))
        manager->UnitKill(this);
    else
        Destroy();
}
