// Oleg Petruny proprietary.

#pragma once

#include "GameFramework/Actor.h"

#include "AgeOfWarUnit.generated.h"

USTRUCT(BlueprintType)
struct FAgeOfWarUnitStats
{
    GENERATED_BODY()

    UPROPERTY(EditAnywhere, BlueprintReadWrite)
    FText name = FText::GetEmpty();

    UPROPERTY(EditAnywhere, BlueprintReadWrite)
    int32 price = 100;

    UPROPERTY(EditAnywhere, BlueprintReadWrite)
    int32 health = 5;

    UPROPERTY(EditAnywhere, BlueprintReadWrite)
    float moveSpeed = 1.0f;

    UPROPERTY(EditAnywhere, BlueprintReadWrite)
    int32 attackDamage = 1;

    UPROPERTY(EditAnywhere, BlueprintReadWrite)
    int32 attackDistance = 1;

    UPROPERTY(EditAnywhere, BlueprintReadWrite)
    float attackRate = 1.0f;

    UPROPERTY(EditAnywhere, BlueprintReadWrite)
    int32 unitSize = 1;

};

UCLASS(Blueprintable, MinimalAPI, Abstract)
class AAgeOfWarUnit : public AActor
{
    GENERATED_BODY()

public:
    enum class UnitTeam
    {
        Player,
        AI
    };

    UFUNCTION(BlueprintPure)
    const FAgeOfWarUnitStats& GetStats();

    class AAgeOfWarManager* manager = nullptr;
    const FAgeOfWarUnitStats* stats = nullptr;
    UnitTeam team;

protected:
    virtual void BeginPlay() override;
    virtual void EndPlay(const EEndPlayReason::Type EndPlayReason) override;
    virtual void Tick(float deltaTime) override;

    void Damage(const int32 damage);

    AAgeOfWarUnit* forwardUnit = nullptr;

private:
    int32 health;
    FTimerHandle attackTimer;
    double lastAttackTimestamp = 0;

};

